;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author : Gregor Mhler (moehler@ims.uni-stuttgart.de)     */
;/*             Date   : last modified 1/30/98                            */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* The German phone set uses the following features:                     */
;/* \begin{itemize}                                                       */
;/* \item vowel/consonant                                                 */
;/* \item vowel length (short, long, diphthong, schwa)                    */
;/* \item vowel height (high, mid, low)                                   */
;/* \item vowel frontness (front, mid, back)                              */
;/* \item vowel openness                                                  */
;/* \item consonant type (stop, fricative, affricate, nasal, liquid,      */
;/* biphonemic)                                                           */
;/* \item place of articulation (labial, labio-dental, dental, alveolar,  */
;/* palatal, velar, uvular)                                               */
;/* \item consonant voicing                                               */
;/* \end{itemize}                                                         */
;/*                                                                       */
;/*=======================================================================*/

;
;  The definition of a reduced german celex phone set 
;

(defPhoneSet
  german

  ;  Phone Features
  (
    ; vowel or consonant
    (vc + -)  
    
    ; vowel length: short long dipthong nasal schwa
    (vlng s l d a n 0)
    
    ; vowel height: high mid low
    (vheight 1 2 3 -)
    
    ; vowel frontness: front mid back
    (vfront 1 2 3 -)
    
    ; vowel openness
    (vopen + -)
    
    ; consonant type: stop fricative affricate nasal liquid 
    (ctype s f a n l 0)
    
    ; place of articulation: labial alveolar palatal labio-dental
    ;                        dental uvular velar 
    (cplace l a p b d u v 0)
    
    ; consonant voicing
    (cvox + -)
  )
  
  ; Phone set members
  (
    (6   +   a   -   -   -   0   0   +)
    (@   +   a   2   2   -   0   0   +)
    (i:  +   l   1   1   -   0   0   +)
    (I   +   s   1   1   -   0   0   +)
    (y:  +   l   1   2   -   0   0   +)
    (Y   +   s   1   2   -   0   0   +)
    (e:  +   l   2   1   -   0   0   +)
    (E   +   s   2   1   -   0   0   +)
    (E:  +   l   2   1   +   0   0   +)
    (2:  +   l   2   2   -   0   0   +)
    (9   +   s   2   2   -   0   0   +)
    (u:  +   l   1   3   -   0   0   +)
    (U   +   s   1   3   -   0   0   +)
    (o:  +   l   2   3   -   0   0   +)
    (O   +   s   2   3   -   0   0   +)
    (a:  +   l   3   2   -   0   0   +)
    (a   +   s   3   2   -   0   0   +)
    (aI  +   d   3   1   -   0   0   +)
    (OY  +   d   2   3   -   0   0   +)
    (aU  +   d   3   2   -   0   0   +)
    (a^  +   n   3   2   -   0   0   +)
    (E^  +   n   2   1   +   0   0   +)
    (o^  +   n   2   3   -   0   0   +)
    (p   -   0   -   -   -   s   l   -)
    (b   -   0   -   -   -   s   l   +)
    (t   -   0   -   -   -   s   a   -)
    (d   -   0   -   -   -   s   a   +)
    (k   -   0   -   -   -   s   p   -)
    (g   -   0   -   -   -   s   p   +)
    (f   -   0   -   -   -   f   b   -)
    (v   -   0   -   -   -   f   b   +)
    (s   -   0   -   -   -   f   a   -)
    (z   -   0   -   -   -   f   a   +)
    (S   -   0   -   -   -   f   p   -)
    (Z   -   0   -   -   -   f   p   +)
    (x   -   0   -   -   -   f   u   -)
    (C   -   0   -   -   -   f   v   -)
    (h   -   0   -   -   -   f   v   -)
    (pf  -   0   -   -   -   a   l   -)
    (ts  -   0   -   -   -   a   a   -)
    (tS  -   0   -   -   -   a   p   -)
    (m   -   0   -   -   -   n   l   +)
    (n   -   0   -   -   -   n   d   +)
    (N   -   0   -   -   -   n   v   +)
    (l   -   0   -   -   -   l   d   +)
    (R   -   0   -   -   -   l   u   +)
    (j   -   0   -   -   -   l   p   +)
    (_   -   0   -   -   -   0   0   -)
    (?   -   0   -   -   -   0   0   -)
  )
)


(PhoneSet.silences '(_))


(defPhoneSet
  german_sampa
  
  ;  Phone Features
  (
    ; vowel or consonant
    (vc + -)  
    
    ; vowel class: plain diphtong semivowel/glide schwa
    (vclass p d g s n -)
    
    ; vowel length: short long
    (vlng s l -)
    
    ; vowel height: high mid1 mid2 low
    (vheight 1 2 3 4 -)
    
    ; vowel frontness: front mid back
    (vfront 1 2 3 -)
    
    ; vowel tenseness
    (vtense + -)
    
    ;vowel rounding
    (vround + -)
    
    ; consonant type: stop fricative affricate nasal liquid glides 
    (ctype s f a n l j -)
    ; place of articulation: labial alveolar palato-alveolar palatal 
    ;                        labio-dental dental velar glottal 
    (cplace p s S C f T x h -)
    
    ; consonant voicing
    (cvox + -)
    
    ; sonorant
    (cson + -)
    
    ; continuant
    (ccont + -)
    
    ; aspirated
    (caspir + -)
  )
  
  ; Phone set members
  (
    ;   VOW CLA LEN HEI FRON TEN ROU CTYP POA VOI SON CONT ASP  
    (6   +   s   s   3   2    -   -   -    -   +   +   +    -)
    (@   +   s   s   2   2    -   -   -    -   +   +   +    -)
    (i:  +   p   l   1   1    +   -   -    -   +   +   +    -)
    (I   +   p   s   1   1    -   -   -    -   +   +   +    -)
    (y:  +   p   l   1   2    +   +   -    -   +   +   +    -)
    (Y   +   p   s   1   2    -   +   -    -   +   +   +    -)
    (e:  +   p   l   2   1    +   -   -    -   +   +   +    -)
    (E   +   p   s   2   1    -   -   -    -   +   +   +    -)
    (E:  +   p   l   2   1    +   -   -    -   +   +   +    -)
    (2:  +   p   l   2   2    +   +   -    -   +   +   +    -)
    (9   +   p   s   2   2    -   +   -    -   +   +   +    -)
    (9:  +   p   l   2   2    -   +   -    -   +   +   +    -)
    (u:  +   p   l   1   3    +   +   -    -   +   +   +    -)
    (U   +   p   s   1   3    -   +   -    -   +   +   +    -)
    (o:  +   p   l   2   3    +   +   -    -   +   +   +    -)
    (O   +   p   s   2   3    -   +   -    -   +   +   +    -)
    (O:  +   p   l   2   3    -   +   -    -   +   +   +    -)
    (a:  +   p   l   3   2    +   -   -    -   +   +   +    -)
    (a   +   p   s   3   2    -   -   -    -   +   +   +    -)
    (aI  +   d   l   3   1    -   -   -    -   +   +   +    -)
    (OY  +   d   l   2   3    -   -   -    -   +   +   +    -)
    (aU  +   d   l   3   2    -   -   -    -   +   +   +    -)
    (eI  +   d   l   3   1    -   -   -    -   +   +   +    -)
    (w   -   -   -   -   -    -   -   j    C   +   +   +    -)
    (i   +   g   s   1   1    -   -   -    -   +   +   +    -)
    (u   +   g   s   1   3    -   -   -    -   +   +   +    -)
    (o   +   g   s   2   3    -   -   -    -   +   +   +    -)
    (e   +   p   s   2   1    +   -   -    -   +   +   +    -)
    (y   +   p   s   1   2    +   +   -    -   +   +   +    -)
    (a~  +   n   l   3   2    +   +   -    -   +   +   +    -)
    (9~  +   n   l   2   2    +   +   -    -   +   +   +    -)
    (e~  +   n   l   2   1    +   -   -    -   +   +   +    -)
    (p   -   -   -   -   -    -   -   s    p   -   -   -    -)
    (b   -   -   -   -   -    -   -   s    p   +   +   -    -)
    (t   -   -   -   -   -    -   -   s    s   -   -   -    -)
    (d   -   -   -   -   -    -   -   s    s   +   +   -    -)
    (k   -   -   -   -   -    -   -   s    x   -   -   -    -)
    (g   -   -   -   -   -    -   -   s    x   +   +   -    -)
    (f   -   -   -   -   -    -   -   f    f   -   -   +    -)
    (v   -   -   -   -   -    -   -   f    f   +   +   +    -)
    (s   -   -   -   -   -    -   -   f    s   -   -   +    -)
    (z   -   -   -   -   -    -   -   f    s   +   +   +    -)
    (S   -   -   -   -   -    -   -   f    S   -   -   +    -)
    (Z   -   -   -   -   -    -   -   f    S   +   +   +    -)
    (x   -   -   -   -   -    -   -   f    x   -   -   +    -)
    (C   -   -   -   -   -    -   -   f    C   -   -   +    -)
    (h   -   -   -   -   -    -   -   f    h   -   -   +    +)
    (T   -   -   -   -   -    -   -   f    T   -   -   +    -)
    (D   -   -   -   -   -    -   -   f    T   +   +   +    -) 
    (pf  -   -   -   -   -    -   -   a    f   -   -   +    -)
    (ts  -   -   -   -   -    -   -   a    s   -   -   +    -)
    (tS  -   -   -   -   -    -   -   a    S   -   -   +    -)
    (dZ  -   -   -   -   -    -   -   a    S   +   +   +    -) 
    (m   -   -   -   -   -    -   -   n    p   +   +   +    -)
    (n   -   -   -   -   -    -   -   n    s   +   +   +    -)
    (N   -   -   -   -   -    -   -   n    C   +   +   +    -)
    (l   -   -   -   -   -    -   -   l    S   +   +   +    -)
    (R   -   -   -   -   -    -   -   f    x   -   -   +    -)
    (j   -   -   -   -   -    -   -   j    C   +   +   +    -)
    (_   -   -   -   -   -    -   -   -    -   -   -   -    -)
    (?   -   -   -   -   -    -   -   s    h   -   -   -    -)
  )
)


(PhoneSet.silences '(_))


(set! german_sampa2german_mbrola_map
  '(("eI" . "e:") 
    ("o"  . "o:") 
    ("?"  . "_" ) ; map glottal stops to pauses
    ("w"  . "v" )
    ("i"  . "i:")
    ("u"  . "u:")
    ("y"  . "y:")
    ("e"  . "e:")
    ("O:" . "O" )
    ("9:" . "9" )
    ("9~" . "E^")
    ("a~" . "a^")
    ("e~" . "E^")
    ("T"  . "s" )
    ("D"  . "z" )
    ("dZ" . "tS")))


(provide 'ims_german_phones)
