;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Gregor Mhler (moehler@ims.uni-stuttgart.de)    */
;/*             Date   :  September 1997                                  */
;/* modified 7/9/98 by Bettina Saeuberlich for festival 1.3               */
;/* modified 11/18/99 by Antje Schweitzer:                                */
;/*  german_end_ged and german_end_punc improved                          */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* Phrasing is handled by Phrasify_German if the parameter               */
;/* "Phrasify_Method" is set to "Phrasify_German".                        */
;/*                                                                       */
;/* Phrasify_German first checks if there are pre-specified tones on any  */
;/* of the words in the utterance (i.e. if any of the words has a feature */
;/* "tone" specifying a boundary tone). In this case, we assume that we   */
;/* want to assign breaks exclusively where they have been specified.     */
;/* Therefore, phrase_cart_tree (which will later be used for             */
;/* prediction of breaks, as "Phrase_Method" is "cart_tree" for all       */
;/* German voices presently defined) is reset to                          */
;/* german_phrase_cart_tree_from_tobi.                                    */
;/*                                                                       */
;/* Otherwise, the default german_phrase_cart_tree as defined in          */
;/* ims_german_phrasify will be used to predict phrasing, unless it       */
;/* is reset (in ims_german_voices) to some more elaborate tree.          */
;/*                                                                       */
;/*=======================================================================*/



(define (Phrasify_German utt)
  (cond
   ((and (or (equal? 'Words (utt.type utt))
             (equal? 'Tokens (utt.type utt)))
         (tobi_tone_prespecified utt))
    (set! phrase_cart_tree_save phrase_cart_tree)
    (set! phrase_cart_tree german_phrase_cart_tree_from_tobi) 
    (Classic_Phrasify utt)
    (set! phrase_cart_tree phrase_cart_tree_save))
   (t (Classic_Phrasify utt))))

(define (tobi_tone_prespecified utt)
  (let ((tobi_found nil)
	(words (utt.relation.items utt 'Word)))
    
    (while (and words (not tobi_found))
; feature "tone" might be prespecified on words or tokens, AS 05/29/00
	   (if (feat.exist (car words) 'tone)
	       (set! tobi_found t)
; if Token relation exists, check tokens as well
               (if (not (null (item.parent (item.relation (car words) 'Token))))
                   (if (feat.exist (item.parent (item.relation (car words) 'Token)) 'tone)
                       (set! tobi_found t)
                       (set! words (cdr words)))
                   (set! words (cdr words)))))
    tobi_found))

;; if one token is related to more than one word, we have to check this
;; for phrase breaks prediction
(define (german_end_punc word)
  "(german_end_punc WORD)
   returns punctuation at end of related Token if WORD is last word
   in Token."
  (if (null (item.next word))
      (item.feat word 'R:Token.parent.punc)
      (if (equal? (item.relation.parent word 'Token) 
		  (item.relation.parent (item.next word) 'Token))
	  "0"
	  (item.feat word 'R:Token.parent.punc))))

(define (german_end_ged word)
  "(german_end_ged WORD)
    returns gedankenstrich at end of related Token if WORD is last word
   in Token."
  (if (null (item.next word))
      (item.feat word 'R:Token.parent.n.name)
  (if (equal? (item.relation.parent word 'Token) 
	      (item.relation.parent (item.next word) 'Token))
      "0"  
      (if (or(string-equal (item.feat word 'R:Token.parent.n.name)"-")
	     (string-equal (item.feat word 'R:Token.parent.n.name)"--"))
	  "-"  ; don't return name of next item, if it's not a -
	  "0"))))

(set! german_phrase_cart_tree_from_tobi
      '
      ((lisp_last_word_in_token is 1)
       ((tone matches "[H\\|L]?%")
        ((BB))
        ((tone matches "-")
         ((B))
         ((R:Token.parent.tone matches "[H\\|L]?%")
          ((BB))
          ((R:Token.parent.tone matches "-")
           ((B))
           ((NB))))))
       ((NB))))

(set! german_phrase_cart_tree
      '
      ((lisp_german_end_punc matches ".*[:;!?].*")
       ((BB))
       ((lisp_german_end_punc matches ".*\\..*")
	((R:Token.parent.token_pos is ordinal)
	 ((NB))
	 ((BB)))
	((lisp_german_end_punc matches ".*\\,.*")
	 ((B))
	 ((lisp_german_end_ged is "-")
	  ((B))
	  ((NB)))))))


(define (last_word_in_token word)
"(last_word_in_token WORD)
returns 1 if word is last word in corresponding token"

(if
 (null (item.next word))
 "1"
 (if (equal? (item.parent (item.relation word 'Token))
             (item.parent (item.relation (item.next word) 'Token)))
     "0"
     "1")))


 ; ganz schoen alt:
;((lisp_german_end_punc matches ".*[?.,:;].*")
; ((lisp_german_end_punc matches ".*[.].*")
;  ((R:Token.parent.token_pos is ordinal)
;   ((NB))
;   ((BB)))
;  ((B)))
; ((n.name is 0)
;  ((B))
;  ((lisp_german_end_ged  is "-")
;   ((B))
;   ((NB))))))


(provide 'ims_german_phrasify)




