;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file handles German token-to-word rules.                         */
;/*                                                                       */
;/*=======================================================================*/

; bugs removed. Martin Barbisch, Ursula Vollmer [16082000] [21082000]


(require 'ims_german_token_to_words_tools)
(require 'ims_patternmatch)
(require 'ims_siodfuncs)
(require 'ims_german_token_to_words_lists)

(set! markdebug nil)

;;tokenpos: 
;time 
;year
;abbr
;abbr?
;comb_abbr
;date
;money
;ordinal
;cardinal
;teilungszahl
;dezimalbruch
;masseinheit
;verhaeltnis
;phonenumber
;card_struc
;bruch


(define (german_token_to_words token name)
  "(german_token_to_words token name)
   is the german version of the token-to-word-rules and  handles
   abbreviations from the Duden and numeral expressions such as
   phone numbers, ratios, dates, currencies, compounds of letters 
   and digits, roman numbers, lines, scale units."

;(format t "GERMANTTW: %s %l\n" name (item.features token))

;  (if markdebug 
;      (begin 
;	(print `****german_token_to_words:)
;(utt.save utt "-")))

  (cond

   ;;abbreviations nach duden-regel wie
   ;;Ver.-Ges.-Vert. (=Versicherungs Gesellschafts Vertreter)
   ;;muessen vor Aufsplittung von '[-/\"]' behandelt werden.
   ;;Token die ein '.-' enthalten sind kombinierte Abkuerzungen (Vers.-Ges.)  
   ((string-matches name "[A-Za-z]+\\(\\.-[A-Za-z]+\\)+")
    (item.set_feat token "punc" 
			 (string-after 
			  (item.feat token "punc") "."))
    (item.set_feat token "token_pos" "comb_abbr")
    (ger_lookup_comb_abbr name))
  
  
   ;; ebenso (Teilungszahlen) , telefonnummern und verhaeltniszahlen.

;;***************************************************************
;;Teilungszahlen (Brche) wurden herausgenommen da zu oft Konflikte mit 
;;Schreibweise wie Im WS 1997/98
;;Hier sollte eventuell eine homograph-disambiguierung vorgeschaltet werden
;   ((pattern-matches name "{[1-9][0-9]*}/{[0-9]+}")  ;;Brche wie '22/34'
;    (item.set_feat token "token_pos" "teilungszahl")
;    (if markdebug (begin (print (list "teilungszahl" name)) (terpri)))
;    (if (string-equal name "1/2")
;	(list "ein" "halb")
;	(append (let ((x (german_parse_cardinal #1)))
;		  ;;geht eventuell auch eleganter...
;		  (if (equal? x `(eins))
;		      (list "ein")
;		      x))
;		(german_parse_fractal #2))))
;;********************************************************************


   ((and (string-matches name "[0-9][0-9]:[0-9][0-9]")     ;;Uhrzeit mit Doppelpkt
         (or (string-matches (item.feat token "n.name") "Uhr")
	     (string-matches (item.feat token "n.name") "h")))
     (item.set_feat token "token_pos" "time")

     ; say "zehn Uhr" instead of "zehn Uhr null" [MB_16082000]
     (append (let ((h   (german_parse_cardinal (string-before name ":"))))
	       (if (equal? h `(eins))
		   (list "ein")
		   h))
	     '("Uhr") 
	     (if (not (equal? (string-after name ":") "00"))
		 (german_parse_cardinal (string-after name ":"))))
   )


   ((string-matches name "[0-9][0-9]:[0-9][0-9]:[0-9][0-9]")     ;;Zeitangabe
    (item.set_feat token "token_pos" "time")
    (append
     (german_parse_cardinal (string-before name ":"))
     '("Stunden")
     (german_parse_cardinal (string-before (cadr (split_string name ":")) ":"))
     '("Minuten")
     '("und")
     (german_parse_cardinal (string-after (cadr (split_string name ":")) ":"))
     '("Sekunden")))


;;-------------------------------------------------------------
;; Waehrungskram, nicht sehr gut


;;   ((string-matches name "[0-9]+,-")
;;    (item.set_feat token "token_pos" "money")
;;    (append
;;     (german_parse_cardinal (string-before name ","))
;;     '("Mark")))

;;   ((string-matches name "\\(DM\\|\\$\\)")
;;    (item.set_feat token "token_pos" "curry")
;;    (item.set_feat (item.next token) "token_pos" "money_mid") 
;;    nil)


   ((and (string-matches name currencies)
	 (and (not (item.next token))    ;; falls naechstes token = nil
	      (not (item.prev token))))    ;; und vorheriges token = nil
    (item.set_feat token "token_pos" "curry")
    (german_fetch_currency2 name))


   ;; currency only in context with numbers
   ((and (string-matches name currencies)
	 (not (item.next token))    ;; falls naechstes token = nil
	 (not (string-matches (item.feat token "p.name") "[0-9]+.*"))) ;; und token davor keine Zahl
;    (item.set_feat token "token_pos" "curry")
;    (german_fetch_currency2 name))
    (list name))    

   ((and (string-matches name currencies)
	 (not (item.prev token))     ;; falls vorheriges token = nil
	 (not (string-matches (item.feat token "n.name") "[0-9]+.*")))  ;; und naechstes token keine Zahl
    (item.set_feat token "token_pos" "curry")
    (german_fetch_currency2 name))


   ((and (string-matches name "[0-9]+")
	 (string-equal (item.feat token "punc") ",-"))
    ;; wird als Geldbetrag betrachtet ! egal ob Waehrung im Kontext
    (item.set_feat token "token_pos" "money_after")
    (item.remove_feature token "punc")
    (cond
     ((string-equal (item.feat token "p.token_pos") "curry")
      (append
       (german_parse_cardinal name)
       (german_fetch_currency2 (item.feat token "p.name"))))
     ((string-matches (item.feat token "n.name") currencies)
      (german_parse_cardinal name))
     (t 
      (append
       (german_parse_cardinal name)
       '("Mark")))))
    

  
   ((and (string-matches name currencies)
	 (or (string-matches (item.feat token "p.name") "[0-9]+.*")
	     (string-matches (item.feat token "n.name") "[0-9]+.*")))
    ;; ist es eine Waehrung?
    (item.set_feat token "token_pos" "curry")

    ;; kommt die Waehrung nach dem Betrag?
    (cond
     ((string-equal (item.feat token "p.token_pos") "money_mid")
	nil)
     ((string-equal (item.feat token "p.token_pos") "money_after")
	(german_fetch_currency2 name))
    
    ;; kommt die Waehrung vor dem Betrag?
     ((string-matches (item.feat token "n.name") "[0-9]+[,][0-9][0-9][0-9]+")
	(item.set_feat (item.next token) "token_pos" "money_after")
	nil)
     ((string-matches (item.feat token "n.name") "[0-9]+[,][0-9][0-9]")
	(item.set_feat (item.next token) "token_pos" "money_mid")
	nil)
     ((string-matches (item.feat token "n.name") "[0-9]+")
	(item.set_feat (item.next token) "token_pos" "money_after")
	nil)
     (t nil)))
    


   ;; ist das vorherige item eine Waehrung?
   ((and (string-equal (item.feat token "p.token_pos") "curry")
	 (string-matches name "[0-9]+.*"))
    (cond
     ((string-equal (item.feat token "token_pos") "money_mid")
	(append
	 (german_parse_cardinal (string-before name ","))
	 (german_fetch_currency2 (item.feat token "p.name"))
	 (german_parse_cardinal (string-after name ","))))
    
     ((string-equal (item.feat token "token_pos") "money_after")
      (cond
       ((string-matches name "[0-9]+")
	(append
	 (german_parse_cardinal name)
	 (german_fetch_currency2 (item.feat token "p.name"))))
       ((string-matches name "[0-9]+[,][0-9][0-9][0-9]+")
	(append
	 (german_parse_cardinal (string-before name ","))
	 '("Komma")
	 (german_parse_charlist (string-after name ","))
	 (german_fetch_currency2 (item.feat token "p.name"))))))
     (t nil)))


   ;; kommt danach eine Waehrung?
   ((and (string-matches name "[0-9]+[,][0-9][0-9][0-9]+")
	 (string-matches (item.feat token "n.name") currencies))
    (item.set_feat token "token_pos" "money_after")
    (append
     (german_parse_cardinal (string-before name ","))
	 '("Komma")
	 (german_parse_charlist (string-after name ","))))


   ((and (string-matches name "[0-9]+[,][0-9][0-9]")
	 (string-matches (item.feat token "n.name") currencies))
    (item.set_feat token "token_pos" "money_mid")
    (append
     (german_parse_cardinal (string-before name ","))
	 (german_fetch_currency2 (item.feat token "n.name"))
	 (german_parse_cardinal (string-after name ","))))

   ((and (string-matches name "[0-9]+")
	 (string-matches (item.feat token "n.name") currencies))
    (item.set_feat token "token_pos" "money_after")
    (german_parse_cardinal name))


;;------------------------------------------------------------------------


   ((string-matches name "[0-9]+:[0-9]+")     ;;Verhaeltnis wie '3:1'
    (item.set_feat token "token_pos" "verhaeltnis")
    (append
     (german_parse_cardinal (string-before name ":"))
     '("zu")
     (german_parse_cardinal (string-after name ":"))))
 

   ((string-matches name "[1-9][0-9]*/[1-9][0-9]*")
    (item.set_feat token "token_pos" "bruch")
    (append
      (german_parse_cardinal (string-before name "/"))
      (german_parse_fractal (string-after   name "/"))))


   ((string-matches name "-")  ;; nice try, funktioniert nicht besonders gut
    (if (and (string-matches (item.feat token "p.name") "[0-9]*")
	     (string-matches (item.feat token "n.name") "[0-9]*")
	     (or (string-matches (item.feat token "n.n.name") "[-|=|+|\\*|-|:]")
		 (string-matches (item.feat token "p.p.name") "[-|=|+|\\*|-|:]")))
    (list "minus")))
;    (list "bis"))) ; changed

   ((string-matches name "\\*")
    (if (and (string-matches (item.feat token "p.name") "[0-9]*")
	     (string-matches (item.feat token "n.name") "[0-9]*"))
	(list "mal")))


   ((string-matches name ":")
    (if (and (string-matches (item.feat token "p.name") "[0-9]*")
	     (string-matches (item.feat token "n.name") "[0-9]*"))
	(list "geteilt" "durch")))
	     

  ;; Numbers beginning with a "0" and containing a '/' or '-' are phonenumbers!??
   ((string-matches name "0[0-9]+[/-][0-9]+")     ;;Telefonnummern
    (item.set_feat token "token_pos" "phonenumber")
    (pattern-matches name "{0[0-9]+}{[/-]}{[0-9]+}")
    (append 
      (german_parse_charlist (string-before name #2))
      (german_parse_charlist (string-after  name #2))))
   

   ;;first we have to split combinations with "-" like 610-Mark-Jobs and the grabage-" like in "Heide"-Moerder"
  ((pattern-matches name "{[^-\"]*}{[-\"]}{.*}")
   (item.set_feat token "token_pos" "comb_abbr")
   (let ((h1 #1)
	 (h3 #3)) ;; patter-match-variable sichern
;     (if (and (pattern-matches h1 "[0-9]+")(pattern-matches h3 "[0-9]+"))
;	 (german_numbers_to_words token name)
     (append      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token h1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))

  ((pattern-matches name "{[^/]+}{/}{.*}")
   (let ((h1 #1)
	 (h3 #3)) ;; patter-match-variable sichern
     (append      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token h1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))
       

  ;; if the token begins with a digit , german_numbers_to_words should
  ;; handle it
  ;; ausgenommen die Faelle, die schon vorher abgefangen wurden ;-)

  ((string-matches name "[0-9].*")
   (german_numbers_to_words token name))

   ;; if previous token was time or money the token shouldn't
   ;; appear in the word-relation
   ;; for example "12.30 Uhr" (if previous name was a time-string) 
   ((and (string-equal (item.feat token "p.token_pos") "time")
   	 (string-matches (item.feat token "p.name") 
   			 ".*[0-9]"))
    nil)

    

    
;;   ((string-equal (item.feat token "p.token_pos") "money_a")
;;    nil)
;;   ((string-equal (item.feat token "n.token_pos") "money_p")
;;    nil)

;;   ((and (string-equal (item.feat token "p.token_pos") "money")
;;	 (string-matches (item.feat token "p.name") ".*[\\.\\|,][0-9][0-9]"))
;;    nil)   



   ;; Zusammensetzungen wie B52, Jger90 ... 
   ((pattern-matches name "{[A-Za-z]+}{[0-9]+}")
    (append (list #1) (german_parse_cardinal #2)))
   

   ;; Roman numerals
;   ((string-matches name "\\(II?I?\\|IV\\|VI?I?I?\\|IX\\|X[VIX]*\\)")
   ((string-matches name "[MDCLXVI]+")
    (cond 
     ((or (string-equal "1" (ger_tok_koenig_names token))
	  (string-equal "1" (ger_tok_koenig token)))
      (item.set_feat token "token_pos" "ordinal")
      (item.set_feat token "punc" ".")
      ;;das ist natrlich nicht korrekt aber besser als nix? 
      (append (if (string-equal (item.feat token "p.name") "der")
		  nil
		  '("der"))
	      (german_numbers_to_words token (ger_tok_roman_to_numstring name))))
     ((or (string-equal "1" (ger_tok_koenigin_names token))
	  (string-equal "1" (ger_tok_koenigin token)))
      (item.set_feat token "token_pos" "ordinal")
      (item.set_feat token "punc" ".")
      ;;das ist natrlich nicht korrekt aber besser als nix? 
      (append  (if (string-equal (item.feat token "p.name") "die")
		  nil
		  '("die"))
               (german_numbers_to_words token (ger_tok_roman_to_numstring name))))
     ((string-matches name "M.+")
      (german_numbers_to_words token (ger_tok_roman_to_numstring name)))

;; AS: Hack for I in English titles
     ; is this smartkom-mode?
     ((and (and (not (null (symbol-bound? 'synthesis_method)))
		(string-equal "Smartkom" synthesis_method))
	   ; is this an I contained in a noun (film title)?
	   (and (and (string-equal "I" name)
		     (string-equal "N" (item.feat (item.parent (item.parent (item.relation token 'Concept))) 'syntaxCategory))))
	   ; are there several words associated with the same lexical item? (multi-word token)
	   (or (and (not (null (item.next token)))
		    (equal? (item.parent (item.parent (item.relation token 'Concept)))
			    (item.parent (item.parent (item.relation (item.next token) 'Concept)))))
	       (and (not (null (item.prev token)))
		    (equal? (item.parent (item.parent (item.relation token 'Concept)))
			    (item.parent (item.parent (item.relation (item.prev token) 'Concept)))))))
      ; just return the name (I) as it is
      (list name))
;     (t (german_parse_charlist name))))
     (t (append
	 '(rmisch)
	 (german_numbers_to_words token (ger_tok_roman_to_numstring name))))))
    
   ;; lines
   ((string-matches name "_____+")
    (list "Linie" "von" "Unterstrichen"))
   ((string-matches name "=====+")
    (list "Linie" "von" "Gleichheitszeichen"))
   ((string-matches name "-----+")
    (list "Linie" "von" "Bindestrichen"))
   ((string-matches name "\\*\\*\\*\\*\\*+")
    (list "Linie" "von" "Sternen"))

  ;; symbols-> splice together or split into 2 words 
  ((string-matches name ".+\".+") 
   (append (german_token_to_words token (string-before name "\"")) 
	   (german_token_to_words token (string-after name "\""))))

  ;;name contains symbols that where spoken
  ;;additional symbols has to be added in ger_symbol_tab too
  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}{.*}")  
   (let ((h2 #2)
	 (h3 #3)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2)
     (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg))))) 
    
  ;;the same but at end of token (".*" don't matcht "" !)
  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}")  
   (let ((h2 #2)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2))))
       
  
  ;;name contains symbols that where not spoken
  ((pattern-matches name "{[^-():;,]*}{[-():;,]}{.*}")
   (let ((h2 #2)
	 (h3 #3)) ;; patter-match-variable sichern
     (append      
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))

  ;;the same but at end of token (".*" don't matcht "" !)
  ;;name contains symbols that where not spoken
  ((pattern-matches name "{[^-():;,]*}{[-():;,]}")
   (let ((h2 #2)) ;; patter-match-variable sichern
     (append      
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))

  
  ;; masseinheiten werden nur erkannt wenn zuvor eine Zahl erkannt wurde
  ((and (pattern-matches name  ger_masseinheit_teststring)
	(member_string (item.feat token "p.token_pos") '("ordinal" "cardinal" "teilungszahl" "dezimalbruch" "bruch")))
   (item.set_feat token "token_pos" "masseinheit") 
   (append (ger_abbr_lookup #1 ger_abbr_masseinheiten_dim_tab)
	   (ger_abbr_lookup #2 ger_abbr_masseinheiten_tab)))

  ((and (pattern-matches name  ger_masseinheit_teststring2)
	(member_string (item.feat token "p.token_pos") '("ordinal" "cardinal" "teilungszahl" "dezimalbruch" "bruch")))
   (item.set_feat token "token_pos" "masseinheit")
   (ger_abbr_lookup #1 ger_abbr_masseinheiten_tab))


   
   ;; abbreviations
   ((let ((exp (ger_check_for_abbr token name)))
      exp))
	  
   ;;"-" kann bindestrich, gredankenstrich, minus ... sein
   ((string-matches name "-") 
    ;; Minus wenn Zahlen vorher und danach
    (if (and (string-matches  (item.feat token "p.name") "[0-9]*") 
	     (string-matches  (item.feat token "n.name") "[0-9]*"))
	(list "minus")
	;; Gedankenstrich sonst:
	;; Aus irgend eimem Grund darf am Anfang/Ende einer Utterance im Wort-relation kein
	;; Leerer String stehen. Bildet man aber im innern eines Wort-relation ein 
	;; Token auf nil ab wird es beim Phrasify micht bercksichtigt...
	;; deshalb die komplizierte Konstruktion
	(if (or (string-equal(item.feat token "p.name") 0)
		(string-equal(item.feat token "n.name") 0))
	    nil
	    (string-equal(item.feat token "p.name") 0)
	    (begin 
	      (item.set_feat token "punc" ".") ; Punkt fr Phrasify setzen
	      (list "")))))
   ;((string-matches name "\\.") NIL)
   ;  (if (or (string-equal(item.feat  token "p.name") 0)
   ;	  (string-equal(item.feat  token "n.name") 0))
   ;      nil
   ;      (begin 
   ;	;	 (item.set_feat token "punc" ".") ; Punkt fr Phrasify setzen
   ;	(list "")))) 
  
  ((pattern-matches name "{[^'`]*}{['`]}{.*}")  
   (german_token_to_words token (string-append #1 #3)))

  ;;Tokens ending with a "." at the end can be abbreviations
  ;;ACHTUNG! diese Abfrage darf nicht nach weiter vorn geschoben werden!
  ((string-matches (item.feat token "punc") "\\..*")
   (let ((uebersetzung (ger_abbr_lookup name ger_abbr_at_eos_tab)))
     (if uebersetzung 
	 uebersetzung
	 (begin 
	   (set! uebersetzung (ger_abbr_lookup name ger_abbr_with_point_tab))
	   (if uebersetzung 
	       (begin
		 (if (not (member_string (item.feat token "n.name") 
					 german_satzanfang))
		     (item.set_feat token "punc" 
					  (string-after 
					   (item.feat token "punc") ".")))
		 uebersetzung)
	       (if (string-matches name "[A-Za-z]*") ;;nur wenn nur noch alphanumerische 
		   (list name)                              ;;Zeichen->word=token
		   (german_parse_charlist name)))))))
  
  ((string-matches name "[A-Za-z][a-z]*[A-Z]+[a-z]*")   ;; wenn Mitten im Wort Grossbuchstaben stehen --> buchstabieren
   (item.set_feat token "token_pos" "abbr")
   (german_parse_charlist name))



  ((string-matches name "[A-Za-z]*") ;;nur wenn nur noch alphanumerische Zeichen->word=token
   (list name))    

  (t               ;;else: buchstabieren (wenn alles schiefgeht)
   (german_parse_charlist name))))
    

;;
;(define german_genus_classification_tree
;  '((n.lisp_token_pos_guess is month)
;    ((Masc))
;    ((p.name in (die Die))
 ;     ((Fem))
;     ((p.name in (der am Der Am))
;      ((Masc))
;      ((p.name in (Das das))
;       ((Neut))
;	((p.name in (Die die))
;	 ((Fem))
;	 ((Masc))))))))

;(define german_casus_classification_tree
;  '((n.lisp_token_pos_guess is month)
;    ((Masc))
;    ((p.name is die)
;     ((Fem))
;     ((p.name in (der))
;      ((Masc))
;      ((p.name is das)
;       ((Neut))
;	((p.name is Die)
;	 ((Fem))
;	 ((Masc))))))))

;(define german_numerus_classification_tree
;  '((n.lisp_token_pos_guess is month)
;    ((Masc))
;    ((p.name is die)
;     ((Fem))
;     ((p.name in (der am ))
;      ((Masc))
;      ((p.name is das)
;       ((Neut))
;	((p.name is Die)
;	 ((Fem))
;	 ((Masc))))))))



(define (german_numbers_to_words token name)
  "(german_numbers_to_words token name) this function deals (hopefully) 
   with tokens beginning with a number. It returns a list of words.
   (blocks of three, compositions of digits and letters, years, dates, 
   clock time, numbers with comma, numbers with period, ordinal numbers, 
   cardinals with leading zeros, numbers in blocks of three with periods, 
   no currencies.)"

;(format t "GERMANNumbersToWords: %s %l\n" name (item.features token))

     ;;called by the cond-branch "roman numerals" and "word starting with a digit" in "german_token_to_words"

  ;; if numbers are written in 3 blocks with single space -> combine them 
  ;; null token if last tokens were already numbers 
  (cond 
   ((and (string-matches name "[0-9][0-9][0-9]") 
	     (or (string-equal 
		  (item.feat token "p.tokenpos") "card_struc")
		 (string-equal 
		  (item.feat token "pp.tokenpos") "card_struc")
		 (and
		   (string-equal 
		    (item.feat token "ppp.tokenpos") "card_struc")
		   (string-matches
		    (item.feat token "p.name") "[0-9][0-9][0-9]") 
                   (string-matches
		    (item.feat token "pp.name") "[0-9][0-9][0-9]") )))
    (if markdebug (begin (print (list "already" name)) (terpri)))
    nil)

   ;;four blocks
   ((and (string-matches 
	  (item.feat token "n.name") "[0-9][0-9][0-9]") 
	 (string-matches 
	  (item.feat token "nn.name") "[0-9][0-9][0-9]") 
	 (string-matches 
	  (item.feat token "nnn.name") "[0-9][0-9][0-9]") 
	 (string-equal (item.feat token "n.whitespace") " ")
	 (string-equal (item.feat token "nn.whitespace") " ")
	 (string-equal (item.feat token "nnn.whitespace") " "))
    (begin 
      (item.set_feat token "tokenpos" "card_struc")
      (german_parse_cardinal 
       (string-append 
	name 
	(item.feat token "n.name")
	(item.feat token "nn.name")
	(item.feat token "nnn.name")))))
   
;;three blocks
   ((and (string-matches 
	  (item.feat token "n.name") "[0-9][0-9][0-9]") 
	 (string-matches 
	  (item.feat token "nn.name") "[0-9][0-9][0-9]") 
	 (string-equal (item.feat token "n.whitespace") " ")
	 (string-equal (item.feat token "nn.whitespace") " "))
    (begin 
      (item.set_feat token "tokenpos" "card_struc")
      (german_parse_cardinal 
       (string-append 
	name 
	(item.feat token "n.name")
	(item.feat token "nn.name")))))

   ;;two blocks
   ((and (string-matches 
	  (item.feat token "n.name") "[0-9][0-9][0-9]") 
	 (string-equal (item.feat token "n.whitespace") " ")
    (begin 
      (item.set_feat token "tokenpos" "card_struc")
      (german_parse_cardinal 
       (string-append 
	name 
	(item.feat token "n.name"))))))
      

   ; Zusammensetzungen like 8fach, 10malig, 16pfnder ...
   ((pattern-matches name "{[0-9]+}{[A-Za-z]+}") 
    (if markdebug (begin (print (list "zusammengesetzt" name)) (terpri)))
    (item.set_feat token "token_pos" "cardinal")
    (let ((h1 #1)
          (h2 #2))
       (cond
         ((pattern-matches h2 ger_masseinheit_teststring)
           (set! h2 (append (ger_abbr_lookup #1 ger_abbr_masseinheiten_dim_tab)
	                    (ger_abbr_lookup #2 ger_abbr_masseinheiten_tab))))
         ((pattern-matches h2 ger_masseinheit_teststring2)
           (set! h2 (ger_abbr_lookup #2 ger_abbr_masseinheiten_tab)))
         (t
           (set! h2 (list h2))))

       (append (german_numbers_to_words token h1) h2))) 
   

   ; years 
   ((and (pattern-matches name "{1[1-9]}00")
	 (member_string (item.feat token "token_pos") '("year" "cardinal")))
    (if markdebug (begin (print (list "years1" name)) (terpri)))
    (append (german_parse_cardinal #1)
	    (list "Hundert")))
   
   ((and (pattern-matches name "{1[1-9]}{[0-9][0-9]}")
	 (member_string (item.feat token "token_pos") '("year" "cardinal")))
    (if markdebug (begin (print (list "years2" name)) (terpri)))
    (append (german_parse_cardinal #1)
	    (list "Hundert")
	    (german_parse_cardinal #2)))


   ;; Date
   ;; 11.12.97 ...
   ((pattern-matches name "{[0-3]?[0-9]}{\\.\\|-}{[0-1]?[0-9]}{\\.\\|-}{[0-9][0-9]}")
    (if markdebug (begin (print (list "date1" name)) (terpri)))
    (item.set_feat token "token_pos" "date")
    (append (german_parse_ordinal 
	     #1
	    (car 
	     (wagon token german_ordinal_prediction_tree)))       
	    (german_parse_ordinal 
	     #3 
	     (car (wagon token german_ordinal_prediction_tree)))  
	   (german_parse_cardinal #5)))
   
  ;;13.4.1995 16-04-1065...
   ((pattern-matches name "{[0-3]?[0-9]}{\\.\\|-}{[0-1]?[0-9]}{\\.\\|-}{[0-9][0-9][0-9][0-9]}")
   (if markdebug (begin (print (list "date1" name)) (terpri)))
   (item.set_feat token "token_pos" "date")
   (append (german_parse_ordinal
	    #1
	    (car 
	     (wagon token german_ordinal_prediction_tree)))       
	   (german_parse_ordinal 
	    #3 
	    (car (wagon token german_ordinal_prediction_tree)))  
	   (if (pattern-matches #5 "{1[1-9]}00")
	       (append (german_parse_cardinal #1)
		       (list "Hundert"))
	       (if (pattern-matches #5 "{1[1-9]}{[0-9][0-9]}")
		   (append (german_parse_cardinal #1)
	       		   (list "Hundert")
			   (german_parse_cardinal #2))
		   (german_parse_cardinal #5)))))
  
  ;; 13.04 15.7. ...   
  ((and (pattern-matches name "{[0-3]?[0-9]}.{[0-1]?[0-9]}.")
	(string-matches (item.feat token "punc") "\\..*"))
   (if markdebug (begin (print (list "date2" name)) (terpri)))
   (item.set_feat token "token_pos" "date")
   (append (german_parse_ordinal 
	    #1 
	    (car 
	     (wagon token german_ordinal_prediction_tree)))       
	   (german_parse_ordinal 
	    #2 
	    (car (wagon token german_ordinal_prediction_tree)))  
	   (german_parse_cardinal #3)))
  
  ;; Time 
  ;; 12.30 Uhr
  ((and (pattern-matches name "{[0-2]?[0-9]}{\\.\\|:}{[0-6][0-9]}")
	(member_string (item.feat token "n.name") `("Uhr" "h")))  
   (if markdebug (begin (print (list "time1" name)) (terpri)))
   (item.set_feat token "token_pos" "time")
   (append (let ((x (german_parse_cardinal #1)))
	     ;;geht eventuell auch eleganter...
	     (if (equal? x `(eins))
		 (list "ein")
		 x))
	   '("Uhr") 
	   (if (not (equal? #3 "00")) (german_parse_cardinal #3))))
   
  ;; 12.30Uhr 12:30h ...
  ((pattern-matches name "{[0-2]?[0-9]}{\\.\\|:}{[0-6][0-9]}{h\\|Uhr}")
   (if markdebug (begin (print (list "time2" name)) (terpri)))
   (item.set_feat token "token_pos" "time")
   (append (let ((x (german_parse_cardinal #1)))
	     ;;geht eventuell auch eleganter...
	      (if (equal? x `(eins))
		  (list "ein")
		  x))
	   '("Uhr") 
	   (if (not (equal? #3 "00")) (german_parse_cardinal #3))))
  

  ;; Money
  ;; hoffentlich haben die token_pos_cart_trees richtig getagt..
  ;;like "12,50 DM"
;;  ((and (string-equal (item.feat token "token_pos") "money_a")
;;        (pattern-matches name "{[1-9][0-9]*}[,|.]{[0-9][0-9]}"))
;;   (if markdebug (begin (print (list "money_a" name)) (terpri)))
;;   (append (german_parse_cardinal #1)
;;           (german_fetch_currency token "n.name")
;;           (german_parse_cardinal #2)
;;           ))
  ;;like "DM 12,50" 
;;  ((and (string-equal (item.feat token "token_pos") "money_p")
;;        (pattern-matches name "{[1-9][0-9]*}[,|.]{[0-9][0-9]}"))
;;   (if markdebug (begin (print (list "money_p" name)) (terpri)))
;;   (append (german_parse_cardinal #1)
;;           (german_fetch_currency token "p.name")
;;           (german_parse_cardinal #2)
;;           ))



  
  ;; numbers with comma seperator
  ((pattern-matches name "{[0-9]+},{[0-9]+}")
   (if markdebug (begin (print (list "comma seperator" name)) (terpri)))
   (item.set_feat token "token_pos" "dezimalbruch")
   (append (german_parse_cardinal #1)
	   (list "Komma")
	   (german_parse_charlist (symbol->string #2))))

  ;; numbers with punkt seperator
  ((pattern-matches name "{[0-9]+}.{[0-9]+}")
   (if markdebug (begin (print (list "comma seperator" name)) (terpri)))
   (item.set_feat token "token_pos" "dezimalbruch")
   (append (german_parse_cardinal #1)
	   (list "Punkt")
	   (german_parse_charlist (symbol->string #2))))


  ;; numbers without a comma
  ;; numbers with a "." -> OrdnungsZahl?
  ;; Ordnungszahlen:
  ;;   Masc/Fem . Nom/Akk/Dat/Gen . Sg/Pl?
  
  ((and (string-matches name "[0-9]+")
	(string-equal (item.feat token "punc") "."))
   (item.set_feat token "token_pos" "ordinal")
    (german_parse_ordinal name
			  (car (wagon token german_ordinal_prediction_tree))))



  ((and (string-matches name "[0-9]+")
	(and 
	 (string-matches (item.feat token "punc") "\\..*")
	 (string-equal (item.feat token "token_pos") "ordinal")))
   (begin
     (if markdebug (begin (print (list "ordnungszahl" name)) (terpri)))
     (item.set_feat token "punc" 
			  (string-after 
			   (item.feat token "punc") "."))
     (german_parse_ordinal name (car (wagon token german_ordinal_prediction_tree)))))    


  ;; Cardinal with leading zeros
  ((pattern-matches name "{0+}{[1-9][0-9]*}")
   (append ; bis zu drei nullen werden ausgesprochen
    (cond ((string-equal #1 "0") '("null"))
	  ((string-equal #1 "00") '("null" "null")) ;z.B. Agent 007
	  ((string-equal #1 "000") '("null" "null" "null")))
    (german_parse_cardinal #2)))
  

  ((string-matches name "[0-9]+")
    (if markdebug (begin (print (list "cardinal" name)) (terpri)))
    (german_parse_cardinal name))


  ;;numbers in three-blocks with "." - seperator
  ((string-matches 
    name 
    "[0-9]?[0-9]?[0-9]\\.\\([0-9][0-9][0-9]\\.\\)*[0-9][0-9][0-9]")
   (german_parse_cardinal (splice_string name ".")))
  ;;else


  ;;name contains symbols that where spoken
  ;;additional symbols has to be added in ger_symbol_tab too
  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}{.*}")  
   (let ((h2 #2)
	 (h3 #3)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2)
     (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))     

  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}{.*}")  
   (let ((h2 #2)
	 (h3 #3)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2)
     (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))     
  ;;the same but at end of token (".*" don't matcht "" !)
  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}")  
   (let ((h2 #2)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2))))
  ;;the same but at end of token (".*" don't matcht "" !)
  ((pattern-matches name "{[^!@#$%^&*=~:?><\/|+_:]*}{[!@#$%^&*=~:?><\/|+_:]}")  
   (let ((h2 #2)) ;; patter-match-variable sichern
     (append 
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (german_parse_1char h2))))

  ((pattern-matches name "{[^/]*}{[/]}{.*}")
   (let ((h2 #2)
	 (h3 #3)) ;; patter-match-variable sichern
     (append      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg))
      (let ((erg (german_token_to_words token h3)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))
       
  ;;the same but at end of token (".*" don't matcht "" !)
  ;;name contains symbols that where not spoken
  ((pattern-matches name "{[^-():;,]*}{[-():;,]}")
   (let ((h2 #2)) ;; patter-match-variable sichern
     (append      
      ;; Die Abfrage auf '(nil) ist umstaendlich und aufwendig aber noetig!?
      (let ((erg (german_token_to_words token #1)))
	(if (equal? erg '(nil))
	    nil
	    erg)))))

  (t
   (if markdebug (begin (print (list "true" name)) (terpri)))
   (append 
    (let ((x  (item.feat token "prepunctuation")))
      (if (member_string x '("" "0")) nil
	  (german_parse_charlist x))) 
    (german_parse_charlist name)
     (let ((x  (item.feat token "punc")))
       (if (member_string x '("" "0")) nil
	   (german_parse_charlist x)))))))


(provide 'ims_german_token_to_words)








