;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file contains all tables, lists and trees used in the            */
;/* token-to-words rules.                                                 */
;/*                                                                       */
;/*=======================================================================*/

; minor bugs removed. Martin Barbisch, Ursula Vollmer [21082000]

;; In dieser Datei sind alle Tabellen, Listen und Bume gespeichert,
;; welche vom Benutzer eventuell Angepasst werden mssen/sollen.


(defvar currencies nil
"currencies is a string, that contains all possible currency abbreviations.")

(set! currencies
"\\(DM\\|S\\|sfr\\|FF\\|bfr\\|Y\\|US\\$\\|\\$\\|EUR\\|E\\|Dk\\|TL\\|can\\$\\|kan\\$\\|dkr\\|\\|L\\|ptas\\|esc\\)")


(defvar ger_abbr_currency_tab nil
"ger_abbr_currency_tab
 is an assoc list that associates currency like $ with the full form
 (Dollar)")
(set! ger_abbr_currency_tab
  '(("DM"   . ("Mark"))
    ("S"    . ("Schilling"))
    ("sfr"  . ("Franken"))
    ("FF"   . ("Franc"))
    ("bfr"  . ("belgische" "Franc"))
    ("Y"    . ("Yen"))
    ("US$"  . ("amerikanische" "Dollar"))
    ("$"    . ("Dollar"))
    ("EUR"  . ("Euro"))
    ("E"    . ("Euro"))
    ("Dk"   . ("Dukaten"))
    ("TL"   . ("Taler"))
    ("can$" . ("kanadische" "Dollar"))
    ("kan$" . ("kanadische" "Dollar"))
    ("dkr"  . ("dnische" "Kronen"))
    (""    . ("Pfund"))
    ("L"    . ("Lire"))
    ("ptas" . ("Pesetas"))
    ("esc"  . ("Escudos"))
    ))


(defvar ger_abbr_without_point_tab nil
 "ger_abbr_without_point_tab 
  contains abbreviations without period.")
(set! ger_abbr_without_point_tab 
  '(
    ;;Titel
    ("Dr" . ("Doktor"))
    ("Prof" . ("Professor"))
    ("Hr" . ("Herr"))
    ("Fr" . ("Frau"))
    ("Frl" . ("Fraeulein"))
    ("Mr" . ("Mister"))
    ("Mrs" . ("Misses"))

    ;;Strassennamen
    ("Str" . ("Strasse"))
    ("Blvd" . ("Boulevard"))
    ("Ave" . ("Avenue"))

    ;;Politik
    ("NATO" . ("Nato"))
    ("NASA" . ("Nasa"))
    ("USA" . ("U" "S" "A"))
    ("US" . ("U" "S"))
    ("GAL" . ("grn" "alternative" "Liste"))
    ("SPD" . ("S" "P" "D"))
    ("CDU" . ("C" "D" "U"))
    ("CSU" . ("C" "S" "U"))
    ("FDP" . ("F" "D" "P"))
    ("REP" . ("Rep"))
    ("RAF" . ("R" "A" "F"))
    ("UNO" . ("Uno"))
    ("UN" . ("U" "N"))
    ("EU" . ("E" "U"))
    ("WEU" . ("W" "E" "U"))
    ("PKK" . ("P" "K" "K"))
    ("DDR" . ("D" "D" "R"))
    ("OB" . ("O" "B"))

    ;;Zeitungen/Medien
    ("TAZ" . ("Taz"))
    ("FAZ" . ("F" "A" "Z"))
    ("ZDF" . ("Z" "D" "F"))
    ("ARD" . ("A" "R" "D"))
    ("RTL" . ("R" "T" "L"))
    ("UKW" . ("U" "K" "W"))

    ;;Umweltschutz
    ("CO" . ("C" "O"))
    ("PNV" . ("" "P" "N" "V"))
    ("BUND" . ("Bund" "fr" "Umwelt" "und" "Naturschutz" "Deutschland"))

    ;;Wirtschaft/Organisationen
    ("AG" . ("A" "G"))
    ("AgV" . ("A" "g" "V"))
    ("AGF" . ("A" "G" "F"))
    ("EC" . ("E" "C"))
    ("GmbH" . ("G" "m" "b" "H"))
    ("KG" . ("K" "G"))
    ("MBROLA" . ("MBROLA")) ; pron. is in the lexicon
    ("IMS" . ("I" "M" "S"))
    ("SmartKom" .("Smartkom"))

     ;;Sport
    ("ATP" . ("eje" "Ti" "Pi"))
    ("WM" . ("W" "M"))
 
    ;;Autos
    ("VW" . ("V" "W"))
    ("BMW" . ("B" "M" "W"))
    ("ADAC" . ("A" "D" "A" "C"))
    ("ACD" . ("A" "C" "D"))
    ("ACE" . ("A" "C" "E"))

    ;;Misc
    ("MEZ" . ("mitteleuropische" "Zeit"))
    ("WS" . ("Wintersemester"))
    ("SS" . ("Sommersemester"))
    ("SOS" . ("S" "O" "S"))
    ))


(defvar ger_abbr_with_point_tab nil
  "ger_abbr_with_point_tab
  is an assoc list that associates abbreviations  with the full form.
  Abbreviations with a period at the end (which is not listed here).")

(set! ger_abbr_with_point_tab
      '(("Abk". ("Abkrzung"))
        ("b.z.w". ("beziehungsweise"))
	("bzw". ("beziehungsweise"))
	("ca" . ("zirka"))
	("Ca" . ("zirka"))
	("d.h" . ("das" "heit"))
	("dh" . ("das" "heit"))
	("o.g" . ("oben" "genannten"))
	("e.t.c" . ("und" "so" "weiter"))
	("etc" . ("und" "so" "weiter"))
	("u.s.w" . ("und" "so" "weiter"))
	("usw" . ("und" "so" "weiter"))
        ("u.U" . ("unter" "Umstnden"))
	("z.B" . ("zum" "Beispiel"))
	("zB" . ("zum" "Beispiel"))
	("Z.B" . ("zum" "Beispiel"))
	("ZB" . ("zum" "Beispiel"))
	("vergl" . ("vergleiche"))
	("Vergl" . ("vergleich"))
	("vgl" . ("vergleiche"))
	("Vgl" . ("vergleich"))
	("Ges" . ("Gesellschaft"))
	("Nr" . ("Nummer"))
	("e.V" . ("e" "V"))    
	("Co" . ("Ko"))
	("Az" . ("Akten" "Zeichen"))
	("Mio" . ("Millionen"))
	("Mrd" . ("Milliarden"))
	("Jan" . ("Januar"))
	("Feb" . ("Februar"))
	("Mrz" . ("Mrz"))
	("Apr" . ("April"))
	("Mai" . ("Mai"))
	("Jun" . ("Juni"))
	("Jul" . ("Juli"))
	("Aug" . ("August"))
	("Sep" . ("September"))
	("Okt" . ("Oktober"))
	("Nov" . ("November"))
	("Dez" . ("Dezember"))
	;new entries
	("Bsp" . ("Beispiel"))
	("frz" . ("franzsisch"))
	("i.A" . ("im" "Allgemeinen"))
	("a.D" . ("auer" "Dienst"))
	("o.B.d.A" . ("ohne" "Beschrnkung" "der" "Allgemeinheit"))

	;;Titel
	("Dr" . ("Doktor"))
	("Prof" . ("Professor"))
	("Hr" . ("Herr"))
	("Fr" . ("Frau"))
	("Frl" . ("Fraeulein"))
	("Mr" . ("Mister"))
	("Mrs" . ("Misses"))

	;;Strassennamen
	("Str" . ("Strasse"))
	("Blvd" . ("Boulevard"))
	("Ave" . ("Aeveniu"))
	
	;    ("" . ("" "" ""))
	;    ("" . ("" "" ""))
	))



(defvar ger_abbr_at_eos_tab nil
"ger_abbr_at_eos_tab
 is an assoc-list of abbreviations that occur almost ever at the end of a 
 sentence. points at after this abbr. are recognized as periods.")
(set! ger_abbr_at_eos_tab
      '(
	("etc" . ("ezetera"))
	("u.s.w" . ("und" "so" "weiter"))
	("usw" . ("und" "so" "weiter"))
	("u.v.m" . ("und" "vieles" "mehr"))
	("uvm" . ("und" "vieles" "mehr"))
	("u.a" . ("und" "andere"))
	("u." . ("und" "hnliche"))
	("u.a.m" . ("und" "andere" "mehr"))
	("u..m" . ("und" " hnliche" "mehr"))
	("u.dgl" . ("und" "dergleichen"))
	("u.dgl.m" . ("und" "dergleichen" "mehr"))
	("u.desgl" . ("und" "desgleichen"))
	("u.desgl.m" . ("und" "desgleichen" "mehr"))
	("o." . ("oder" "hnliches"))
	("o..m" . ("oder" "hnliches" "mehr"))
	("u.d." . ("und" "dem" "hnliche"))
	;    ("" . (""""""))
	;    ("" . (""""""))
	))



;; Achtung! Abkrzungen, welche in den nchsten beiden Tabellen
;; eingetragen werden mssen auch in den beiden Variablen
;; ger_masseinheit_teststring und ger_masseinheit_teststring2
;; eingetragen werden.  leider nicht sonderlich elegant... sollte in
;; zukuenftigen versionen verbessert werden.
(defvar ger_abbr_masseinheiten_tab nil
  "ger_abbr_masseinheiten_tab
   is an assoc list that associates units of measurement like g with the 
   full form (Gramm)")

(set! ger_abbr_masseinheiten_tab
      '(
	("A"   . ("Ampere"))
	("eV"  . ("Elektronenvolt"))
        ("C"  . ("Grad" "Celsius"))
	("C"   . ("Coulomb"))
	("F"   . ("Farad"))
	("g"   . ("Gramm"))
	("h"   . ("Stunden"))
	("Hz"  . ("Herz"))
	("H"   . ("Henry"))
	("J"   . ("Joule"))
	("K"   . ("Kelvin"))
	("l"   . ("Liter"))
	("m"   . ("Meter"))
	("N"   . ("Newton"))
	("P"   . ("Pond"))
	("s"   . ("Sekunden"))
	("t"   . ("Tonnen"))
	("W"   . ("Watt"))
	("V"   . ("Volt"))
	("cal" . ("Kalorien"))
	("min" . ("Minuten"))
	("Wh"  . ("Watt" "Stunden"))

        ; new entries, in the right place?
        ("ha"  . ("Hektar"))
        ("qkm" . ("Quadratkilometer"))
        ("qcm" . ("Quadratzentimeter"))
        ("qm"  . ("Quadratmeter"))
        ("ccm" . ("Kubikzentimeter"))
        ("km/h". ("Kilometer" "pro" "Stunde"))
	))


(defvar ger_abbr_masseinheiten_dim_tab nil
"ger_abbr_masseinheiten_dim_tab
contains the dimension of german Masseinheiten")

(set! ger_abbr_masseinheiten_dim_tab
      '(
	("c" . ("zenti"))
	("d" . ("dezi"))
	("h" . ("hekto"))
	("k" . ("kilo"))
	("m" . ("milli"))
	("M" . ("mega"))
;	("u" . ("mikro")) ; um a preposition in most cases
	("" . ("mikro"))
	("G" . ("giga"))
	("n" . ("nano"))
	("T" . ("tera"))
	("p" . ("pico"))
	))


(defvar ger_masseinheit_teststring nil
  "ger_masseinheit_teststring
   Check on membership of the amount of units of measurements.")
(set! ger_masseinheit_teststring  
  "{[cdhkmMGnTp]}{A\\|eV\\|C\\|F\\|g\\|Hz\\|H\\|J\\|K\\|l\\|m\\|N\\|P\\|s\\|V\\|cal\\|Wh\\|W}")


(defvar ger_masseinheit_teststring2 nil
  "ger_masseinheit_teststring2
   Check on membership of the amount of units
   of measurements, but without the dimensions. (milli, hekto ... )")
(set! ger_masseinheit_teststring2 
  "{A\\|eV\\|C\\|C\\|F\\|g\\|h\\|Hz\\|H\\|J\\|K\\|l\\|m\\|N\\|P\\|s\\|t\\|W\\|V\\|cal\\|min\\|Wh\\|ha\\|qkm\\|qcm\\|qm\\|ccm\\|km/h}")


(defvar ger_abbr_months_tab nil
  "ger_abbr_months_tab
   is an assoc list that associates abbreviations of german
   month-names with the full form.
   try: (ger_abbr_lookup \"Jan\" ger_abbr_months_tab")
(set! ger_abbr_months_tab
      '(("Jan" . ("Januar"))
	("Feb" . ("Februar"))
	("Mrz" . ("Mrz"))
	("Maerz" . ("Mrz"))
	("Mrz" . ("Mrz"))
	("Apr" . ("April"))
	("Mai" . ("Mai"))
	("Jun" . ("Juni"))
	("Jul" . ("Juli"))
	("Aug" . ("August"))
	("Sep" . ("September"))
	("Okt" . ("Oktober"))
	("Nov" . ("November"))
	("Dez" . ("Dezember"))))


(defvar ger_abbr_days_tab nil
"ger_abbr_days_tab
 is an assoc list that associates abbreviations of german weekday-names
 with the full form.
 try: (ger_abbr_lookup \"So\" ger_abbr_days_tab")

(set! ger_abbr_days_tab
  '(
    ("Mo" . ("Montag"))
    ("Di" . ("Dienstag"))
    ("Mi" . ("Mittwoch"))
    ("Do" . ("Donnerstag"))
    ("Fr" . ("Freitag"))
    ("Sa" . ("Samstag"))
    ("So" . ("Sonntag"))
))


(defvar ger_symbols_tab nil
"ger_symbols_tab
is an assoc list that contains most symbols like & ^ @ *... and the 
corresponding full form")

(set! ger_symbols_tab
  '(
    ("." . ( "Punkt"))
    ("," .("Komma"))
    (":" .("Doppelpunkt"))
    (">" .("grer" "als"))
    ("<" .("kleiner" "als"))
    ("/" .("Schrgstrich"))
    ("?" .("Fragezeichen"))
    ("+" .("plus"))
    ("-" .("minus"))
    ("_" .("Unterstrich"))
    ("=" .("ist" "gleich"))
    ("$" . ("Dollar"))
    ("%" . ("Prozent"))
    ("" . ("Paragraph"))
    ("#" . ("Doppelkreuz")) 
    ("+" . ("plus")) 
    ("*" . ("Stern")) 
    ("~"  .("Tilde")) 
    ("^"  .("Dach")) 
    ("@"  .("Klammeraffe")) 
    ("&"  .("und")) 
    ("|"  .("senkrechter" "Strich")) 
    ("\\" . ("bckslsch")) 
    ("\"" . ("Anfhrungszeichen"))
    ("{"  .("geschweifte" "Klammer" "auf"))
    ("}"  .("geschweifte" "Klammer" "zu"))
    ("]"  .("eckige" "Klammer" "zu"))
    ("["  .("eckige" "Klammer" "auf"))
    (")"  .("Klammer" "zu"))
    ("("  .("Klammer" "auf"))
    ("!" .("Ausrufezeichen"))
    ("`" .("Quote"))
    ("" .("Quote"))
    ("'" .("Backquote"))
    ("" .("Pfund"))
    ("" .("Grad"))
    ("" .("hoch" "eins"))
    ("" .("hoch" "zwei"))
    ("" .("hoch" "drei"))
    ("" .("plus" "minus"))
    ("" .("ein" "viertel"))
    ("" .("ein" "halb"))
    ("" .("drei" "viertel"))
    ("" .("Copyright"))
))
  

(defvar ger_digits_tab nil
"ger_digits_tab
is an assoc list that contains all german digits (0, 1, 2, 3...) and 
the corresponding full form")

(set! ger_digits_tab
  '(
    ("0" . ("null"))
    ("1" . ("eins"))
    ("2" . ("zwei"))
    ("3" . ("drei"))
    ("4" . ("vier"))
    ("5" . ("fnf"))
    ("6" . ("sechs"))
    ("7" . ("sieben"))
    ("8" . ("acht"))
    ("9" . ("neun"))
))



(defvar ger_char_tab nil
  "ger_char_tab
  assoc list that contains all characters stored in ger_digits_tab and 
  ger_symbols_tab; and the corresponding full form")

(set! ger_char_tab
  (append                    
    ger_digits_tab
    ger_symbols_tab)
)


(defvar ger_abbr_compl_tab
  (append 
   ger_abbr_days_tab 
   ger_abbr_without_point_tab 
   ger_abbr_months_tab 
   ger_abbr_masseinheiten_dim_tab
   ger_abbr_masseinheiten_tab
   ger_abbr_with_point_tab
   ger_abbr_currency_tab))


(defvar german_satzanfang nil
  "german_satzanfang
  list of words that occur only at the beginning of
  german sentences.")
(set! german_satzanfang '(An Am Bei Beim Bereits Das Dem Den Der Die Diese 
 Dieser Dir Du Ein Erst Er Es Euch Frher Heute Hier Im In Ich Ihr
 Mir Nur Schon Seit Sie So Und Uns Vom War Was Wegen Wer Weshalb Wie Wir Wieso
 Wo Zum))



(defvar german_king_title nil
  "german_king_title
  list of words that are german king-like titles.")
(set! german_king_title '(knig kaiser herzog baron king papst graf lord earl prinz koenig))


(defvar german_queen_title nil
  "german_queen_title
  list of words that are german queen-like titles.")
(set! german_queen_title '(knigin kaiserin herzogin baronin ppstin grfin prinzessin koenigin paepstin graefin))


(defvar german_king_name nil
  "german_king_name
  list of words that are german king names.")
(set! german_king_name '(louis henry charles philip george georg edward eduard
			       pius william wilhelm richard john paul peter
			       johannes-paul nikolaus alexander frederick james
			       alfonso ivan napolean leo gregory gregor ludwig
			       otto friedrich karl rio andrew))


(defvar german_queen_name nil
  "german_queen_name
  list of words that are german queen names.")
(set! german_queen_name '(catherine katharina alexandria elizabeth mary maria anne elisabeth kleopatra))


	      
(defvar german_year_indicator nil
  "german_year_indicator
   a list of german words that often stand before years.")
(set! german_year_indicator 
      '(Seit seit bereits Bereits Jahre Jahr
	     Sylvester Erst erst Schon schon Herbst Winter Sommer Frhjahr
	     Frhling bis Bis Ende gegenber Gegenber anno Fruehjahr Fruehling
gegenueber Gegenueber))



;; Mit Hilfe dieses Klassifikationsbaumes kann die Endung einer
;; Ordnungszahl aus dem ihrem engeren Kontext bestimmt werden. Dieser
;; Klassifikationbaum ist von Hand erstellt. Eine andere Mglichkeit
;; wre die automatische erstellung mit Hilfe des wagon-Tools (siehe
;; Festival-Manual) Eine weitere, bessere Strategie die Endungen von zu
;; bestimmen liegt darin, die Endungbestimmung in einem spteren Modul
;; (nach dem POS-Tagging bzw. parsen) durchzufhren, da dann mehr
;; Kontextinformationen bezglich Kasus, Genus und Nummerus zur
;; vefgung stehen. 

(defvar german_ordinal_prediction_tree nil
  "german_ordinal_prediction_tree
   this tree is used to determin the suffix of german ordinals.")


(set! german_ordinal_prediction_tree
      '((n.lisp_token_pos_guess is month) ;;Monatsname danach -> mnnlich
	((p.name in (den dem der des am im zum zur beim jedem vom
			 Den Dem Der Des Am Im Zum Zur Beim Jedem Vom))
	 ((p.name in  (der Der)) ;der 2. April ... 
	  ((e))
	  ((en)))
	 ((p.name is 0) ;;end of utterance
	  ((n.name in ,german_months)
	   ((er))
	   ((ens)))
	  ((er))))
	((p.name in (den dem des am im zum zur beim jedem vom
			 Den Dem Des Am Im Zum Zur Beim Jedem Vom))
	 ((en))	 
	 ((p.name in (der die das diese jene 
			  Der Die Das Diese Jene))
	  ((e))
	  ((token_pos is date)
	   ((er))
	   ((lisp_ger_tok_koenig_names is 1)
	    ((e))
	    ((lisp_ger_tok_koenigin_names is 1)
	     ((e))
	     ((lisp_ger_tok_koenig is 1)
	      ((e))
	      ((lisp_ger_tok_koenigin is 1)
	       ((e))
	       ((p.name is 0) ;;end of utterance
		((n.name in ,german_months)
		 ((er))
		 ((ens)))
		((er))))))))))))


(provide 'ims_german_token_to_words_lists)






