;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher                             */
;/*             Date   :  September 1997                                  */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* Contains a function to match patterns.                                */
;/*                                                                       */
;/*=======================================================================*/

(define (pattern-matches string pattern) 
    "(pattern-matches string pattern) 
    matches string against pattern and returns a list of
    bindings. { and } shouldn't occur in string or pattern!  It only
    finds the longest match of regex!  Regular expression must be put 
    in {}-parantheses.  For each expression in
    {} a Scheme-variable named #1, #2 ... is generated and the part of
    string that matches this expression is bound to this variable
    (similar to patternmatching in PERL). If match succeeds
    pattern-matches returns t, nil otherwise.  Example: (string-match2
    \"123.234\" \"{[0-9]+}.{[0-9]+}\") returns t and the
    variable-bindings (set-symbol-value! #1 123) and
    (set-symbol-value! #2 234).  string-match2 should be used with caution!: 
    all! Variables named #1, #2 ... will be overwritten!"

    (if markdebug 
	(begin 
	  (print `****pattern-matches:)
	  (print (list "string" string))
	  (print (list "pattern" pattern))))
    
    (define (string-match3 string pattern)
      (if markdebug 
	  (begin 
	    (print `****string-match3:)
	    (print (list "string" string))
	    (print (list "pattern" pattern))))
    
      (if (string-equal string pattern) ;;string=pattern => success
	  (begin
	    (resetvars)
	    t  ;;succeed
	    )
	  (if (string-equal (string-car pattern) "{")
	      (let ((patbefore (string-cdr (string-before pattern "}")))
		    (restpat (string-after pattern "}"))
		    (reststring "")
		    (varval nil))
		(set! varval (mark-string-match string patbefore))
		(if (eq? varval nil)
		    (begin
		      (resetvars)
		      nil)  ;;fail
		    (begin
		      (set-symbol-value! (newvar) (symbol->string varval))  ;;returna a string?! Aenderung 1
 		      (set! reststring (mark-string-rest string patbefore))
		      (string-match3 reststring restpat))))
	      (if (string-equal (string-car pattern) (string-car string))
		  (string-match3 (string-cdr string) 
				 (string-cdr pattern)) 
		  (begin
		    (resetvars)
		    nil) ;;fail. Could be there is another match, but only
	          ;;the longest is tried, no backtracking 
		  ))))
    
    
    (define newvar
      (let ((x 0))
	(lambda ()
	  (set! x (+ x 1))
	  (symbolconc '# (list->symbol (symbolexplode x))))))
    
    (define (resetvars) t)

    (string-match3 string pattern))

(provide 'ims_patternmatch)






